{ **************************************************************************** }
{ *                                                                          * }
{ * CHICKY WOGGY (VTech, Electronic Tini-Arcade) Simulator                   * }
{ * by Luca "MADrigal" Antignano, December 2009 (revision: August 2021)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;    im_background_screen_top: TImage;
    im_background_screen:     TImage;

    btn_power_top:            TImage;    btn_power_on:             TImage;
    btn_select_top:           TImage;    btn_select_down:          TImage;
    btn_start_top:            TImage;    btn_start_down:           TImage;

    btn_joystick_top:         TImage;

    btn_ctrlr_overlay_up:     TImage;
    btn_ctrlr_overlay_left:   TImage;
    btn_ctrlr_overlay_right:  TImage;
    btn_ctrlr_overlay_down:   TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen control panel
    im_panel_game_over: TImage;

    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_lives_1: TImage; im_panel_lives_2: TImage; im_panel_lives_3: TImage;

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; im_panel_digit_pos_5: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_chicky_1_1: TImage; im_chicky_2_1: TImage; im_chicky_3_1: TImage; im_chicky_4_1: TImage; im_chicky_5_1: TImage;
    im_chicky_1_2: TImage; im_chicky_2_2: TImage; im_chicky_3_2: TImage; im_chicky_4_2: TImage; im_chicky_5_2: TImage;
    im_chicky_1_3: TImage; im_chicky_2_3: TImage; im_chicky_3_3: TImage; im_chicky_4_3: TImage; im_chicky_5_3: TImage;
    im_chicky_1_4: TImage; im_chicky_2_4: TImage;                        im_chicky_4_4: TImage; im_chicky_5_4: TImage;
    im_chicky_1_5: TImage; im_chicky_2_5: TImage; im_chicky_3_5: TImage; im_chicky_4_5: TImage; im_chicky_5_5: TImage;
    im_chicky_1_6: TImage; im_chicky_2_6: TImage; im_chicky_3_6: TImage; im_chicky_4_6: TImage; im_chicky_5_6: TImage;
    im_chicky_1_7: TImage; im_chicky_2_7: TImage; im_chicky_3_7: TImage; im_chicky_4_7: TImage; im_chicky_5_7: TImage;

    im_egg_1_1:    TImage; im_egg_2_1:    TImage; im_egg_3_1:    TImage; im_egg_4_1:    TImage; im_egg_5_1:    TImage;
    im_egg_1_2:    TImage; im_egg_2_2:    TImage; im_egg_3_2:    TImage; im_egg_4_2_w:  TImage; im_egg_5_2:    TImage;
    im_egg_1_3:    TImage; im_egg_2_3:    TImage; im_egg_3_3:    TImage; im_egg_4_3:    TImage; im_egg_5_3:    TImage;
    im_egg_1_4:    TImage; im_egg_2_4:    TImage;                        im_egg_4_4:    TImage; im_egg_5_4:    TImage;
    im_egg_1_5:    TImage; im_egg_2_5:    TImage; im_egg_3_5:    TImage; im_egg_4_5:    TImage; im_egg_5_5:    TImage;
    im_egg_1_6:    TImage; im_egg_2_6_w:  TImage; im_egg_3_6:    TImage; im_egg_4_6:    TImage; im_egg_5_6:    TImage;
    im_egg_1_7:    TImage; im_egg_2_7:    TImage; im_egg_3_7:    TImage; im_egg_4_7:    TImage; im_egg_5_7:    TImage;

    im_wolf_1_1:   TImage; im_wolf_2_1:   TImage; im_wolf_3_1:   TImage; im_wolf_4_1:   TImage; im_wolf_5_1:   TImage;
    im_wolf_1_2:   TImage; im_wolf_2_2:   TImage; im_wolf_3_2:   TImage; im_wolf_4_2:   TImage; im_wolf_5_2:   TImage;
    im_wolf_1_3:   TImage; im_wolf_2_3:   TImage; im_wolf_3_3:   TImage; im_wolf_4_3:   TImage; im_wolf_5_3:   TImage;
    im_wolf_1_4:   TImage; im_wolf_2_4:   TImage; im_wolf_3_4:   TImage; im_wolf_4_4:   TImage; im_wolf_5_4:   TImage;
                                                  im_wolf_3_4b:  TImage;
    im_wolf_1_5:   TImage; im_wolf_2_5:   TImage; im_wolf_3_5:   TImage; im_wolf_4_5:   TImage; im_wolf_5_5:   TImage;
    im_wolf_1_6:   TImage; im_wolf_2_6:   TImage; im_wolf_3_6:   TImage; im_wolf_4_6:   TImage; im_wolf_5_6:   TImage;
    im_wolf_1_7:   TImage; im_wolf_2_7:   TImage; im_wolf_3_7:   TImage; im_wolf_4_7:   TImage; im_wolf_5_7:   TImage;

                                                  im_gate_3_1:   TImage;
    im_gate_1_4:   TImage;                                                                      im_gate_5_4:   TImage;
                                                  im_gate_3_7:   TImage;

{ * TIMERS                                                                   * }
    Timer_Game:           TTimer;
    Timer_Game_Over:      TTimer;
    Timer_Game_Start:     TTimer;
    Timer_Miss:           TTimer;
    Timer_Power_Pill:     TTimer;
    Timer_Stage_Complete: TTimer;
    Timer_Stage_Start:    TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                 (Sender: TObject);
    procedure FormActivate               (Sender: TObject);
    procedure FormClose                  (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Show_All               (b: boolean);

    procedure Vid_Show_All_Chicky        (b: boolean);
    procedure Vid_Show_All_Egg           (b: boolean);
    procedure Vid_Show_All_Gate          (b: boolean);
    procedure Vid_Show_All_Wolf          (b: boolean);

    procedure Vid_Show_Chicky            (b: boolean);

    procedure Vid_Show_Panel_Lives       (var p: integer);
    procedure Vid_Show_Panel_Score       (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                   (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove     (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_power_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_select_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_select_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_start_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_start_topMouseUp       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_upMouseDown(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseDown(Sender:TObject;Button:TMouseButton;Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseUp(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown(Sender:TObject;Button:TMouseButton;Shift:TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseUp(Sender:TObject;Button:TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseDown(Sender:TObject;Button:TMouseButton;Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseUp(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    
    procedure FormKeyDown                (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                  (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_power_down;
    procedure Inp_select_down;           procedure Inp_select_up;
    procedure Inp_start_down;            procedure Inp_start_up;

    procedure Inp_up_down;               procedure Inp_up_up;
    procedure Inp_left_down;             procedure Inp_left_up;
    procedure Inp_right_down;            procedure Inp_right_up;
    procedure Inp_down_down;             procedure Inp_down_up;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Set_Mode;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Gate;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_GameTimer            (Sender: TObject);
    procedure Timer_Game_OverTimer       (Sender: TObject);
    procedure Timer_Game_StartTimer      (Sender: TObject);
    procedure Timer_MissTimer            (Sender: TObject);
    procedure Timer_Power_PillTimer      (Sender: TObject);
    procedure Timer_Stage_CompleteTimer  (Sender: TObject);
    procedure Timer_Stage_StartTimer     (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup             (Sender: TObject);
    procedure pm_1_aboutClick            (Sender: TObject);
    procedure pm_2_readmeClick           (Sender: TObject);
    procedure pm_3_whatsnewClick         (Sender: TObject);
    procedure pm_4_manualClick           (Sender: TObject);
    procedure pm_6_configClick           (Sender: TObject);
    procedure pm_7_keysClick             (Sender: TObject);
    procedure pm_8_hiscoreClick          (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick     (Sender: TObject);

    procedure la_menu_itemMouseEnter     (Sender: TObject);
    procedure la_menu_itemMouseLeave     (Sender: TObject);
    procedure la_menu_itemMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu      (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Chicky Woggy (VTech, Electronic Tini-Arcade)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY        = 'Software\MADrigal\ChickyWT50';                              // registry key

  HINT_CLICK     = 'click or press';
  HINT_POWER_OFF = 'to turn on the game';
  HINT_POWER_ON  = 'to turn off the game';
  HINT_SELECT    = 'to select game';
  HINT_START     = 'to start game';
  HINT_JOYSTICK  = 'activate the controller overlays or use';

{}MENU_ITEMS = 14;
  NUM_KEYS = 7;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save settings)

{ * sound management * }
  pfs_bonus,
  pfs_complete,
  pfs_gameover,
  pfs_miss,
  pfs_move,
  pfs_power1,
  pfs_power2,
  pfs_poweron,
  pfs_start,
  pfs_stgstart:            PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_score:         array[1..5]       of TImage;                         // panel: empty sprites to host score/time digits
  aIm_panel_numbers:       array[0..9]       of TImage;
  aIm_panel_lives:         array[1..3]       of TImage;                         // panel: lives (1-3)

  aIm_chicky:              array[1..5, 1..7] of TImage;
  aIm_egg:                 array[1..5, 1..7] of TImage;
  aIm_wolf:                array[1..5, 1..7] of TImage;
  aIm_gate:                array[1..5, 1..7] of TImage;

{ * arrays of numbers *}
  aLabyrinth:              array[0..6, 0..8] of record                          // 1..5, 1..7 cells are "visible" labyrinth; others used for additional "invisible" positions
                                                  iContent: 0..2;               // 0=empty; 1=egg; 2=worm
                                                  Chicky,                       // available cells to chicky
                                                  Wolf: record                  // available cells to wolves
                                                          iAllowedDirections: 0..15; // 0=none; 1=up; 2=left; 4=right; 8=down; ...and various combinations 1..15
                                                        end; // record
                                                end; // record

  rChicky:                 record
                             iPosX,
                             iPosY,
                             iDirection: integer;
                           end;

  rWolf:                   array[1..2] of record
                             iPosX,
                             iPosY,
                             iDirection,
                             iStatus:    integer;                               // 0-9: inhouse; 10: attack; 11: escaping
                           end;

{ * arrays of mixed types *}
  aKeys:                   array[1..NUM_KEYS] of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=OFF, 1=ON, etc.
  iScore,                                                                       // 0-99990
  iHiScoreA,                                                                    // 0-99990
  iHiScoreB,                                                                    // 0-99990

  iScoreBonus,                                                                  // needed score for bonus life (2000, etc...)
  iScoreWolf,                                                                   // value for eating a wolf (200 for first, 400 for second)
  iLivesLeft,                                                                   // -1 to 2
  iStage,                                                                       // current stage
  iGate,                                                                        // 1 to 5 (1=clear; 2=top-down; 3=left-right; 4=top-down-right; 5=left);

  iLoop,
  iLoopPowerPill,                                                               // 0 to ... how long "power pill" lasts in terms of game ticks
  iLoopGeneric,                                                                 // used for generic purpose in timers procedures
  iMaxPowerPill,                                                                // how many ticks a powerpill lasts (depends on stage)
  iMaxWolf                                                                      // 1-2 (maximum allowed wolves on screen simultaneously, depends on stage)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bKeyPressed                                                                   // true while left/right arrow keys are being kept pressed
                         : boolean;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_score[1] := im_panel_digit_pos_1; aIm_panel_score[2] := im_panel_digit_pos_2; aIm_panel_score[3] := im_panel_digit_pos_3; aIm_panel_score[4] := im_panel_digit_pos_4; aIm_panel_score[5] := im_panel_digit_pos_5;

  aIm_panel_numbers[0] := im_panel_number_0; aIm_panel_numbers[1] := im_panel_number_1; aIm_panel_numbers[2] := im_panel_number_2; aIm_panel_numbers[3] := im_panel_number_3; aIm_panel_numbers[4] := im_panel_number_4;
  aIm_panel_numbers[5] := im_panel_number_5; aIm_panel_numbers[6] := im_panel_number_6; aIm_panel_numbers[7] := im_panel_number_7; aIm_panel_numbers[8] := im_panel_number_8; aIm_panel_numbers[9] := im_panel_number_9;

  aIm_panel_lives[1] := im_panel_lives_1; aIm_panel_lives[2] := im_panel_lives_2; aIm_panel_lives[3] := im_panel_lives_3;

  // on-screen items
  aIm_chicky[1, 1] := im_chicky_1_1; aIm_chicky[2, 1] := im_chicky_2_1; aIm_chicky[3, 1] := im_chicky_3_1; aIm_chicky[4, 1] := im_chicky_4_1; aIm_chicky[5, 1] := im_chicky_5_1;
  aIm_chicky[1, 2] := im_chicky_1_2; aIm_chicky[2, 2] := im_chicky_2_2; aIm_chicky[3, 2] := im_chicky_3_2; aIm_chicky[4, 2] := im_chicky_4_2; aIm_chicky[5, 2] := im_chicky_5_2;
  aIm_chicky[1, 3] := im_chicky_1_3; aIm_chicky[2, 3] := im_chicky_2_3; aIm_chicky[3, 3] := im_chicky_3_3; aIm_chicky[4, 3] := im_chicky_4_3; aIm_chicky[5, 3] := im_chicky_5_3;
  aIm_chicky[1, 4] := im_chicky_1_4; aIm_chicky[2, 4] := im_chicky_2_4;                                    aIm_chicky[4, 4] := im_chicky_4_4; aIm_chicky[5, 4] := im_chicky_5_4;
  aIm_chicky[1, 5] := im_chicky_1_5; aIm_chicky[2, 5] := im_chicky_2_5; aIm_chicky[3, 5] := im_chicky_3_5; aIm_chicky[4, 5] := im_chicky_4_5; aIm_chicky[5, 5] := im_chicky_5_5;
  aIm_chicky[1, 6] := im_chicky_1_6; aIm_chicky[2, 6] := im_chicky_2_6; aIm_chicky[3, 6] := im_chicky_3_6; aIm_chicky[4, 6] := im_chicky_4_6; aIm_chicky[5, 6] := im_chicky_5_6;
  aIm_chicky[1, 7] := im_chicky_1_7; aIm_chicky[2, 7] := im_chicky_2_7; aIm_chicky[3, 7] := im_chicky_3_7; aIm_chicky[4, 7] := im_chicky_4_7; aIm_chicky[5, 7] := im_chicky_5_7;

  aIm_egg[1, 1] := im_egg_1_1;       aIm_egg[2, 1] := im_egg_2_1;       aIm_egg[3, 1] := im_egg_3_1;       aIm_egg[4, 1] := im_egg_4_1;       aIm_egg[5, 1] := im_egg_5_1;
  aIm_egg[1, 2] := im_egg_1_2;       aIm_egg[2, 2] := im_egg_2_2;       aIm_egg[3, 2] := im_egg_3_2;       aIm_egg[4, 2] := im_egg_4_2_w;     aIm_egg[5, 2] := im_egg_5_2;
  aIm_egg[1, 3] := im_egg_1_3;       aIm_egg[2, 3] := im_egg_2_3;       aIm_egg[3, 3] := im_egg_3_3;       aIm_egg[4, 3] := im_egg_4_3;       aIm_egg[5, 3] := im_egg_5_3;
  aIm_egg[1, 4] := im_egg_1_4;       aIm_egg[2, 4] := im_egg_2_4;                                          aIm_egg[4, 4] := im_egg_4_4;       aIm_egg[5, 4] := im_egg_5_4;
  aIm_egg[1, 5] := im_egg_1_5;       aIm_egg[2, 5] := im_egg_2_5;       aIm_egg[3, 5] := im_egg_3_5;       aIm_egg[4, 5] := im_egg_4_5;       aIm_egg[5, 5] := im_egg_5_5;
  aIm_egg[1, 6] := im_egg_1_6;       aIm_egg[2, 6] := im_egg_2_6_w;     aIm_egg[3, 6] := im_egg_3_6;       aIm_egg[4, 6] := im_egg_4_6;       aIm_egg[5, 6] := im_egg_5_6;
  aIm_egg[1, 7] := im_egg_1_7;       aIm_egg[2, 7] := im_egg_2_7;       aIm_egg[3, 7] := im_egg_3_7;       aIm_egg[4, 7] := im_egg_4_7;       aIm_egg[5, 7] := im_egg_5_7;

  aIm_wolf[1, 1] := im_wolf_1_1;     aIm_wolf[2, 1] := im_wolf_2_1;     aIm_wolf[3, 1] := im_wolf_3_1;     aIm_wolf[4, 1] := im_wolf_4_1;     aIm_wolf[5, 1] := im_wolf_5_1;
  aIm_wolf[1, 2] := im_wolf_1_2;     aIm_wolf[2, 2] := im_wolf_2_2;     aIm_wolf[3, 2] := im_wolf_3_2;     aIm_wolf[4, 2] := im_wolf_4_2;     aIm_wolf[5, 2] := im_wolf_5_2;
  aIm_wolf[1, 3] := im_wolf_1_3;     aIm_wolf[2, 3] := im_wolf_2_3;     aIm_wolf[3, 3] := im_wolf_3_3;     aIm_wolf[4, 3] := im_wolf_4_3;     aIm_wolf[5, 3] := im_wolf_5_3;
  aIm_wolf[1, 4] := im_wolf_1_4;     aIm_wolf[2, 4] := im_wolf_2_4;     aIm_wolf[3, 4] := im_wolf_3_4;     aIm_wolf[4, 4] := im_wolf_4_4;     aIm_wolf[5, 4] := im_wolf_5_4;
  aIm_wolf[1, 5] := im_wolf_1_5;     aIm_wolf[2, 5] := im_wolf_2_5;     aIm_wolf[3, 5] := im_wolf_3_5;     aIm_wolf[4, 5] := im_wolf_4_5;     aIm_wolf[5, 5] := im_wolf_5_5;
  aIm_wolf[1, 6] := im_wolf_1_6;     aIm_wolf[2, 6] := im_wolf_2_6;     aIm_wolf[3, 6] := im_wolf_3_6;     aIm_wolf[4, 6] := im_wolf_4_6;     aIm_wolf[5, 6] := im_wolf_5_6;
  aIm_wolf[1, 7] := im_wolf_1_7;     aIm_wolf[2, 7] := im_wolf_2_7;     aIm_wolf[3, 7] := im_wolf_3_7;     aIm_wolf[4, 7] := im_wolf_4_7;     aIm_wolf[5, 7] := im_wolf_5_7;

                                                                        aIm_gate[3, 1] := im_gate_3_1;
  aIm_gate[1, 4] := im_gate_1_4;                                                                                                              aIm_gate[5, 4] := im_gate_5_4;
                                                                        aIm_gate[3, 7] := im_gate_3_7;

  // labyrinth settings: visible labyrinth
  with aLabyrinth[1, 1] do begin Chicky.iAllowedDirections := 12; {right, down}           Wolf.iAllowedDirections := 12; {right, down}      end;
  with aLabyrinth[1, 2] do begin Chicky.iAllowedDirections := 13; {up, right, down}       Wolf.iAllowedDirections := 13; {up, right, down}   end;
  with aLabyrinth[1, 3] do begin Chicky.iAllowedDirections := 13; {up, right, down}       Wolf.iAllowedDirections := 13; {up, right, down}   end;
  with aLabyrinth[1, 4] do begin Chicky.iAllowedDirections := 11; {up, left, down}        Wolf.iAllowedDirections := 9;  {up, down}         end;
  with aLabyrinth[1, 5] do begin Chicky.iAllowedDirections := 5;  {up, right}             Wolf.iAllowedDirections := 5;  {up, right}        end;
  with aLabyrinth[1, 6] do begin Chicky.iAllowedDirections := 12; {right, down}           Wolf.iAllowedDirections := 12; {right, down}      end;
  with aLabyrinth[1, 7] do begin Chicky.iAllowedDirections := 5;  {up, right}             Wolf.iAllowedDirections := 5;  {up, right}        end;

  with aLabyrinth[2, 1] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 6;  {left, right}      end;
  with aLabyrinth[2, 2] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 6;  {left, right}      end;
  with aLabyrinth[2, 3] do begin Chicky.iAllowedDirections := 14; {left, right, down}     Wolf.iAllowedDirections := 14; {left, right, down} end;
  with aLabyrinth[2, 4] do begin Chicky.iAllowedDirections := 9;  {up, down}              Wolf.iAllowedDirections := 9;  {up, down}         end;
  with aLabyrinth[2, 5] do begin Chicky.iAllowedDirections := 15; {up, left, right, down} Wolf.iAllowedDirections := 7;  {up, left, right}   end;
  with aLabyrinth[2, 6] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 6;  {left, right}      end;
  with aLabyrinth[2, 7] do begin Chicky.iAllowedDirections := 7;  {up, left, right}       Wolf.iAllowedDirections := 6;  {left, right}      end;

  with aLabyrinth[3, 1] do begin Chicky.iAllowedDirections := 15; {up, left, right, down} Wolf.iAllowedDirections := 6;  {left, right}       end;
  with aLabyrinth[3, 2] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 6;  {left, right}       end;
  with aLabyrinth[3, 3] do begin Chicky.iAllowedDirections := 7;  {up, left, right}       Wolf.iAllowedDirections := 6;  {left, right}       end;
  with aLabyrinth[3, 4] do begin Chicky.iAllowedDirections := 0;  {none}                  Wolf.iAllowedDirections := 1;  {up}                end;
  with aLabyrinth[3, 5] do begin Chicky.iAllowedDirections := 14; {left, right, down}     Wolf.iAllowedDirections := 14; {left, right, down} end;
  with aLabyrinth[3, 6] do begin Chicky.iAllowedDirections := 7;  {up, left, right}       Wolf.iAllowedDirections := 7;  {up, left, right}   end;
  with aLabyrinth[3, 7] do begin Chicky.iAllowedDirections := 14; {left, right, down}     Wolf.iAllowedDirections := 6;  {left, right}       end;

  with aLabyrinth[4, 1] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 6;  {left, right}       end;
  with aLabyrinth[4, 2] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 6;  {left, right}       end;
  with aLabyrinth[4, 3] do begin Chicky.iAllowedDirections := 14; {left, right, down}     Wolf.iAllowedDirections := 14; {left, right, down} end;
  with aLabyrinth[4, 4] do begin Chicky.iAllowedDirections := 9;  {up, down}              Wolf.iAllowedDirections := 9;  {up, down}          end;
  with aLabyrinth[4, 5] do begin Chicky.iAllowedDirections := 15; {up, left, right, down} Wolf.iAllowedDirections := 7;  {up, left, right}   end;
  with aLabyrinth[4, 6] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 6;  {left, right}       end;
  with aLabyrinth[4, 7] do begin Chicky.iAllowedDirections := 7;  {up, left, right}       Wolf.iAllowedDirections := 6;  {left, right}       end;

  with aLabyrinth[5, 1] do begin Chicky.iAllowedDirections := 10; {left, down}            Wolf.iAllowedDirections := 10; {left, down}        end;
  with aLabyrinth[5, 2] do begin Chicky.iAllowedDirections := 11; {up, left, down}        Wolf.iAllowedDirections := 11; {up, left, down}    end;
  with aLabyrinth[5, 3] do begin Chicky.iAllowedDirections := 11; {up, left, down}        Wolf.iAllowedDirections := 11; {up, left, down}    end;
  with aLabyrinth[5, 4] do begin Chicky.iAllowedDirections := 13; {up, right, down}       Wolf.iAllowedDirections := 9;  {up, down}          end;
  with aLabyrinth[5, 5] do begin Chicky.iAllowedDirections := 3;  {up, left}              Wolf.iAllowedDirections := 3;  {up, left}          end;
  with aLabyrinth[5, 6] do begin Chicky.iAllowedDirections := 10; {left, down}            Wolf.iAllowedDirections := 10; {left, down}        end;
  with aLabyrinth[5, 7] do begin Chicky.iAllowedDirections := 3;  {up, left}              Wolf.iAllowedDirections := 3;  {up, left}          end;

  // labyrinth settings: invisible labyrinth (through the gates and under the bridges)
  with aLabyrinth[0, 0] do begin Chicky.iAllowedDirections := 9;  {up, down}              Wolf.iAllowedDirections := 0;  {none}              end; // under the top-center bridge
  with aLabyrinth[0, 4] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 0;  {none}              end; // left of the left gate
  with aLabyrinth[2, 8] do begin Chicky.iAllowedDirections := 9;  {up, down}              Wolf.iAllowedDirections := 0;  {none}              end; // under the bottom-left bridge
  with aLabyrinth[3, 0] do begin Chicky.iAllowedDirections := 9;  {up, down}              Wolf.iAllowedDirections := 0;  {none}              end; // over of the top gate
  with aLabyrinth[3, 8] do begin Chicky.iAllowedDirections := 9;  {up, down}              Wolf.iAllowedDirections := 0;  {none}              end; // below of the bottom gate
  with aLabyrinth[4, 8] do begin Chicky.iAllowedDirections := 9;  {up, down}              Wolf.iAllowedDirections := 0;  {none}              end; // under the bottom-right bridge
  with aLabyrinth[6, 4] do begin Chicky.iAllowedDirections := 6;  {left, right}           Wolf.iAllowedDirections := 0;  {none}              end; // right of the right gate

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Power On/Off" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Select"       (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Start"        (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 37);                             // "Up"           (default: "UP")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 39);                             // "Left"         (default: "LEFT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 40);                             // "Right"        (default: "RIGHT")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 38);                             // "Down"         (default: "DOWN")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_power_top.Hint           := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_POWER_OFF; // "Click to turn game on"
  btn_select_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_SELECT;    // "Select"
  btn_start_top.Hint           := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_START;     // "Start"

  btn_joystick_top.Hint        := HINT_JOYSTICK + ' ' + aKeyHints[aKeys[4].int] + ', ' // "Up"
                                                      + aKeyHints[aKeys[5].int] + ', ' // "Left"
                                                      + aKeyHints[aKeys[6].int] + ', ' // "Right"
                                                      + aKeyHints[aKeys[7].int];       // "Down"

  btn_ctrlr_overlay_up.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];   // "Up"
  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];   // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];   // "Right"
  btn_ctrlr_overlay_down.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];   // "Down"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}aMenuItem[12] := pm_12_controller;
{}aMenuItem[13] := pm_13_n3;
{}aMenuItem[14] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bKeyPressed := false;                                                         // no key is currently being kept pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_bonus);    FSOUND_Sample_Free(pfs_complete);
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_miss);
  FSOUND_Sample_Free(pfs_move);     FSOUND_Sample_Free(pfs_power1);
  FSOUND_Sample_Free(pfs_power2);   FSOUND_Sample_Free(pfs_poweron);
  FSOUND_Sample_Free(pfs_start);    FSOUND_Sample_Free(pfs_stgstart);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_up.Visible    := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_down.Visible  := bDisplayCtrlOverlays;

  btn_joystick_top.Visible := (not bDisplayCtrlOverlays);
end;





procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides nearly all sprites (except score panel)                       }
  var
    i: integer;
begin
  im_panel_game_over.Visible := b;
  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  i := Ord(b) * 3; Vid_Show_Panel_Lives(i);

  Vid_Show_All_Chicky(b);
  Vid_Show_All_Egg(b);
  Vid_Show_All_Wolf(b);
  Vid_Show_All_Gate(b);
end;





procedure TForm1.Vid_Show_All_Chicky(b: boolean);
{ displays/hides all chickies                                                  }
  var
    i, j: integer;
begin
  for i := 1 to 5 do
    for j := 1 to 7 do
      if not ((i = 3) and (j = 4)) then aIm_chicky[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Egg(b: boolean);
{ displays/hides all chickies                                                  }
  var
    i, j: integer;
begin
  for i := 1 to 5 do
    for j := 1 to 7 do
      if not ((i = 3) and (j = 4)) then aIm_egg[i,j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Gate(b: boolean);
{ displays/hides all gates                                                     }
begin
  aIm_gate[3, 1].Visible := b;
  aIm_gate[1, 4].Visible := b;
  aIm_gate[5, 4].Visible := b;
  aIm_gate[3, 7].Visible := b;
end;





procedure TForm1.Vid_Show_All_Wolf(b: boolean);
{ displays/hides all wolves                                                    }
  var
    i, j: integer;
begin
  for i := 1 to 5 do
    for j := 1 to 7 do
      aIm_wolf[i, j].Visible := b;

  im_wolf_3_4b.Visible := b;
end;





procedure TForm1.Vid_Show_Chicky(b: boolean);
{ displays/hides chicky as stated on his iPosX and iPosY coordinates           }
begin
  if      ((rChicky.iPosX > 0) and (rChicky.iPosX < 6))
  and     ((rChicky.iPosY > 0) and (rChicky.iPosY < 8))
  and not ((rChicky.iPosX = 3) and (rChicky.iPosY = 4)) then
    aIm_chicky[rChicky.iPosX, rChicky.iPosY].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Lives(var p: integer);
{ displays as many sprites as p on the lives counter (0=nothing visible)       }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_lives[i].Visible := (i <= p);                                     // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 100000 do                                                          // prevents displayed score from being above 99990
    Dec(t, 100000);

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if t > 9999 then aIm_panel_score[1].Picture := aIm_panel_numbers[t div 10000 mod 10].Picture else aIm_panel_score[1].Picture := nil; // tens of thousands
      if t > 999  then aIm_panel_score[2].Picture := aIm_panel_numbers[t div  1000 mod 10].Picture else aIm_panel_score[2].Picture := nil; // thousands
      if t > 99   then aIm_panel_score[3].Picture := aIm_panel_numbers[t div   100 mod 10].Picture else aIm_panel_score[3].Picture := nil; // hundreds
      if t > 9    then aIm_panel_score[4].Picture := aIm_panel_numbers[t div    10 mod 10].Picture else aIm_panel_score[4].Picture := nil; // tens
                       aIm_panel_score[5].Picture := aIm_panel_numbers[t           mod 10].Picture;                                        // units
    end
  else                                                                            // hide
    for i := 1 to 5 do
      aIm_panel_score[i].Picture := nil;

  for i := 1 to 5 do
    aIm_panel_score[i].Visible := b;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_power_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ power button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_power_down;
end;





procedure TForm1.btn_select_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ select button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_select_down;
end;





procedure TForm1.btn_select_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ select button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_select_up;
end;





procedure TForm1.btn_start_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ start button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_start_down;
end;





procedure TForm1.btn_start_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ start button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_start_up;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ up button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ up button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_up_up;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ down button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ down button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_down_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_power_down  else                              // Power On/Off
  if Key = aKeys[2].word then Inp_select_down else                              // Select Game A/B
  if Key = aKeys[3].word then Inp_start_down  else                              // Start

  if Key = aKeys[4].word then Inp_up_down     else                              // Up
  if Key = aKeys[5].word then Inp_left_down   else                              // Left
  if Key = aKeys[6].word then Inp_right_down  else                              // Right
  if Key = aKeys[7].word then Inp_down_down;                                    // Down
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  if Key = aKeys[2].word then Inp_select_up else                                // Select Game A/B
  if Key = aKeys[3].word then Inp_start_up  else                                // Start

  if Key = aKeys[4].word then Inp_up_up     else                                // Up
  if Key = aKeys[5].word then Inp_left_up   else                                // Left
  if Key = aKeys[6].word then Inp_right_up  else                                // Right
  if Key = aKeys[7].word then Inp_down_up;                                      // Down
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_power_down;
{ power button: PRESS                                                          }
begin
  // sets game mode
  if iMode = 0 then
    begin
      Snd_Play(pfs_poweron);

      iMode := 1; Gam_Set_Mode;                                                 // off -> "select game"
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_POWER_ON; // "click to turn off"
    end
  else
    begin
      iMode := 0; Gam_Set_Mode;                                                 // whatever else mode -> off
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_POWER_OFF // "click to turn on"
    end;

  btn_power_on.Visible := (iMode = 1);
end;





procedure TForm1.Inp_select_down;
{ select button: PRESS                                                         }
begin
  btn_select_down.Visible := true;

  case iMode of
    1: begin iMode := 2; Gam_Set_Mode; end;
    2: begin iMode := 3; Gam_Set_Mode; end;
    3: begin iMode := 2; Gam_Set_Mode; end;
  end; // case..of
end;





procedure TForm1.Inp_select_up;
{ select button: RELEASE                                                       }
begin
  btn_select_down.Visible := false;
end;





procedure TForm1.Inp_start_down;
{ start button: PRESS                                                          }
begin
  btn_start_down.Visible := true;

  if (iMode = 2) or (iMode = 3) then
    begin
      Inc(iMode, 2); Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_start_up;
{ start button: RELEASE                                                        }
begin
  btn_start_down.Visible := false;
end;




procedure TForm1.Inp_up_down;
{ up button: PRESS                                                             }
begin
  if (iMode = 6) or (iMode = 7) then
    if Timer_Game.Enabled
    and ((aLabyrinth[rChicky.iPosX, rChicky.iPosY].Chicky.iAllowedDirections and 1) = 1) then
      rChicky.iDirection := 1;                                                  // direction for chicky: up
end;





procedure TForm1.Inp_up_up;
{ up button: RELEASE                                                           }
begin
  bKeyPressed := false;
end;





procedure TForm1.Inp_left_down;
{ left button: PRESS                                                           }
begin
  if (iMode = 6) or (iMode = 7) then
    if Timer_Game.Enabled
    and ((aLabyrinth[rChicky.iPosX, rChicky.iPosY].Chicky.iAllowedDirections and 2) = 2) then
      rChicky.iDirection := 2;                                                  // direction for chicky: left
end;





procedure TForm1.Inp_left_up;
{ left button: RELEASE                                                         }
begin
  bKeyPressed := false;
end;





procedure TForm1.Inp_right_down;
{ right button: PRESS                                                          }
begin
  if (iMode = 6) or (iMode = 7) then
    if Timer_Game.Enabled
      and ((aLabyrinth[rChicky.iPosX, rChicky.iPosY].Chicky.iAllowedDirections and 4) = 4) then
      rChicky.iDirection := 4;                                                  // direction for chicky: right
end;





procedure TForm1.Inp_right_up;
{ right button: RELEASE                                                        }
begin
  bKeyPressed := false;
end;





procedure TForm1.Inp_down_down;
{ down button: PRESS                                                           }
begin
  if (iMode = 6) or (iMode = 7) then
    if Timer_Game.Enabled
      and ((aLabyrinth[rChicky.iPosX, rChicky.iPosY].Chicky.iAllowedDirections and 8) = 8) then
      rChicky.iDirection := 8;                                                  // direction for chicky: down
end;





procedure TForm1.Inp_down_up;
{ down button: RELEASE                                                         }
begin
  bKeyPressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Game.Enabled           := false;
  Timer_Game_Over.Enabled      := false;
  Timer_Game_Start.Enabled     := false;
  Timer_Miss.Enabled           := false;
  Timer_Power_Pill.Enabled     := false;
  Timer_Stage_Complete.Enabled := false;
  Timer_Stage_Start.Enabled    := false;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/ACL mode, depending on the value of the iMode variable  }
  var
    i, j: integer;
begin
  case iMode of
    0: begin                                                                    // OFF
         Gam_Stop_All_Timers;                                                           // stops all timers
         Snd_Silence;                                                                   // stops playing sound samples

         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         i := 0; Vid_Show_Panel_Lives(i);                                               // hides all sprites from misses counter
         i := 0; Vid_Show_Panel_Score(i, false);                                        // hides all sprites from score counter
         im_background_screen.Visible := false;                                         // hides labyrinth
       end;
    1: begin                                                                    // DEMO
         Gam_Stop_All_Timers;                                                           // stops all timers

         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         i := 0; Vid_Show_Panel_Lives(i);                                               // hides all sprites from misses counter
         i := 0; Vid_Show_Panel_Score(i, false);                                        // hides all sprites from score counter
         im_background_screen.Visible := true;                                          // displays labyrinth
       end;
    2, 3: begin                                                                 // GAME SELECT A / B
         Gam_Stop_All_Timers;                                                           // stops all timers

         im_panel_game_a.Visible := iMode = 2;                                          // displays "game A" in mode 2
         im_panel_game_b.Visible := iMode = 3;                                          // displays "game B" in mode 3
       end;
    4, 5: begin                                                                 // HISCORE A / B
         Vid_Show_All(false);                                                           // hides all items, chars, etc.

         case iMode of
           4: begin Vid_Show_Panel_Score(iHiScoreA, true); im_panel_game_a.Visible := true; end;
           5: begin Vid_Show_Panel_Score(iHiScoreB, true); im_panel_game_b.Visible := true; end;
         end; // case..of

         iLivesLeft := 3; Vid_Show_Panel_Lives(iLivesLeft);                             // displays lives
         aIm_egg[2, 6].Visible := true; aIm_egg[4, 2].Visible := true;                  // displays worms

         Snd_Play(pfs_start);                                                           // plays "Start" tune

         iLoopGeneric := 1;                                                             // initializes "Game Start" timer
         Timer_Game_Start.Interval := 2900;
         Timer_Game_Start.Enabled := true;                                              // starts "Game Start" timer
       end;
    6, 7: begin                                                                 // GAME A / B
         iScore := 0; Vid_Show_Panel_Score(iScore, true);                               // displays score (=0)
         iScoreBonus := 2000;
         iLivesLeft := 2;

         for i := 0 to 6 do                                                             // initializes labyrinth
           for j := 0 to 8 do
             aLabyrinth[i, j].iContent := 0;

         for i := 1 to 5 do                                                             // initializes labyrinth
           for j := 1 to 7 do
             aLabyrinth[i, j].iContent := 1;

         aLabyrinth[2, 6].iContent := 2;                                                // initializes labyrinth
         aLabyrinth[4, 2].iContent := 2;                                                // initializes labyrinth
         aLabyrinth[3, 4].iContent := 0;                                                // initializes labyrinth

         iStage := 1; Gam_Set_Difficulty;

         iLoopGeneric := 1;                                                             // initializes "Stage Start" timer
         Timer_Stage_Start.Interval := 1;
         Timer_Stage_Start.Enabled := true;                                             // starts "Game Start" timer
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Difficulty;
{ sets number of wolves on screen simultaneously, value for the TimerGame      }
{ interval, and duration of power-pill, making game faster/slower depending on }
{ which game is played (A or B)                                                }
begin
  case iMode of
    6: begin                                                                    // Game A
         // game speed
         Timer_Game.Interval := 370; {x2}

         // powerpill duration
         case iStage of
                1,  2: iMaxPowerPill := 43; {ok}
                3,  4: iMaxPowerPill := 41; {ok}
                5,  6: iMaxPowerPill := 37; {ok}
                7,  8: iMaxPowerPill := 33; {ok}
                9, 10: iMaxPowerPill := 31; {ok}
           11, 12, 13: iMaxPowerPill := 28; {ok}
                   14: iMaxPowerPill := 25; {ok}
           else        iMaxPowerPill := 22; {ok}
         end; // case..of
       end;
    7: begin                                                                    // Game B
         // game speed
         Timer_Game.Interval := 240; {x2}

         // powerpill duration
         case iStage of
           1, 2: iMaxPowerPill := 29; {ok}
           3, 4: iMaxPowerPill := 27; {ok}
           5, 6: iMaxPowerPill := 23; {ok}
           7, 8: iMaxPowerPill := 21; {ok}
              9: iMaxPowerPill := 19; {??}
           else  iMaxPowerPill := 17; {??}
         end; // case..of
       end;
  end; // case..of

  // maximum wolves on screen for either Game A and Game B
  if iStage = 1 then
    iMaxWolf := 1
  else
    iMaxWolf := 2;
end;





procedure TForm1.Gam_Set_Gate;
{ sets config for gates (visible/not visible)                                  }
begin
  case iGate of
    1: begin                                                                    // all gates OPEN
         aLabyrinth[3, 1].Chicky.iAllowedDirections := 15; {up, left, right, down}
         aLabyrinth[1, 4].Chicky.iAllowedDirections := 11; {up, left, down}
         aLabyrinth[5, 4].Chicky.iAllowedDirections := 13; {up, right, down}
         aLabyrinth[3, 7].Chicky.iAllowedDirections := 14; {left, right, down}
         aIm_gate[3, 1].Visible := false;
         aIm_gate[1, 4].Visible := false;
         aIm_gate[5, 4].Visible := false;
         aIm_gate[3, 7].Visible := false;
       end;
    2: begin                                                                    // up/down CLOSED; left/right OPEN
         aLabyrinth[3, 1].Chicky.iAllowedDirections := 14; {left, right, down}
         aLabyrinth[1, 4].Chicky.iAllowedDirections := 11; {up, left, down}
         aLabyrinth[5, 4].Chicky.iAllowedDirections := 13; {up, right, down}
         aLabyrinth[3, 7].Chicky.iAllowedDirections := 6;  {left, right}
         aIm_gate[3, 1].Visible := true;
         aIm_gate[1, 4].Visible := false;
         aIm_gate[5, 4].Visible := false;
         aIm_gate[3, 7].Visible := true;
       end;
    3: begin                                                                    // up/down OPEN; left/right CLOSED
         aLabyrinth[3, 1].Chicky.iAllowedDirections := 15; {up, left, right, down}
         aLabyrinth[1, 4].Chicky.iAllowedDirections := 9;  {up, down}
         aLabyrinth[5, 4].Chicky.iAllowedDirections := 9;  {up, down}
         aLabyrinth[3, 7].Chicky.iAllowedDirections := 14; {left, right, down}
         aIm_gate[3, 1].Visible := false;
         aIm_gate[1, 4].Visible := true;
         aIm_gate[5, 4].Visible := true;
         aIm_gate[3, 7].Visible := false;
       end;
    4: begin                                                                    // up/down/right OPEN; left OPEN
         aLabyrinth[3, 1].Chicky.iAllowedDirections := 14; {left, right, down}
         aLabyrinth[1, 4].Chicky.iAllowedDirections := 11; {up, left, down}
         aLabyrinth[5, 4].Chicky.iAllowedDirections := 9;  {up, down}
         aLabyrinth[3, 7].Chicky.iAllowedDirections := 6;  {left, right}
         aIm_gate[3, 1].Visible := true;
         aIm_gate[1, 4].Visible := false;
         aIm_gate[5, 4].Visible := true;
         aIm_gate[3, 7].Visible := true;
       end;
    5: begin                                                                    // up/down/right OPEN; left CLOSED
         aLabyrinth[3, 1].Chicky.iAllowedDirections := 15; {up, left, right, down}
         aLabyrinth[1, 4].Chicky.iAllowedDirections := 9;  {up, down}
         aLabyrinth[5, 4].Chicky.iAllowedDirections := 13; {up, right, down}
         aLabyrinth[3, 7].Chicky.iAllowedDirections := 14; {left, right, down}
         aIm_gate[3, 1].Visible := false;
         aIm_gate[1, 4].Visible := true;
         aIm_gate[5, 4].Visible := false;
         aIm_gate[3, 7].Visible := false;
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game timer: everything happens here!                                    }
  var
    i, j, iCount:         integer;
    iTempPosX, iTempPosY,
    iTempDir:             integer;
    iTempScore:           integer;
    iTempWolf,
    iTempWolfStatus:      integer;
    bMistake:             boolean;
    bWolfDirDone:         boolean;
    bPowerPillcontinues:  boolean;
    pfSound:              PFSoundSample;
begin
  bMistake := false;

  if Odd(iLoop) then
{ chicky turn                                                                  }
    begin
      // move chicky
      if rChicky.iDirection <> 0 then
        if (aLabyrinth[rChicky.iPosX, rChicky.iPosY].Chicky.iAllowedDirections and rChicky.iDirection) = rChicky.iDirection then
          begin
            iTempPosX := rChicky.iPosX;
            iTempPosY := rChicky.iPosY;

            case rChicky.iDirection of
              1: if (iTempPosX = 0) and (iTempPosY = 0) then begin iTempPosX := 3; iTempPosY := 1; end else
                 if (iTempPosX = 2) and (iTempPosY = 7) then iTempPosY := 8 else
                 if (iTempPosX = 2) and (iTempPosY = 8) then iTempPosY := 5 else
                 if (iTempPosX = 3) and (iTempPosY = 0) then iTempPosY := 8 else
                 if (iTempPosX = 3) and (iTempPosY = 3) then begin iTempPosX := 0; iTempPosY := 0; end else
                 if (iTempPosX = 4) and (iTempPosY = 7) then iTempPosY := 8 else
                 if (iTempPosX = 4) and (iTempPosY = 8) then iTempPosY := 5 else
                   Dec(iTempPosY);

              2: if (iTempPosX = 0) and (iTempPosY = 4) then iTempPosX := 6 else
                  Dec(iTempPosX);

              4: if (iTempPosX = 6) and (iTempPosY = 4) then iTempPosX := 0 else
                  Inc(iTempPosX);

              8: if (iTempPosX = 0) and (iTempPosY = 0) then begin iTempPosX := 3; iTempPosY := 3; end else
                 if (iTempPosX = 2) and (iTempPosY = 5) then iTempPosY := 8 else
                 if (iTempPosX = 2) and (iTempPosY = 8) then iTempPosY := 7 else
                 if (iTempPosX = 3) and (iTempPosY = 1) then begin iTempPosX := 0; iTempPosY := 0; end else
                 if (iTempPosX = 3) and (iTempPosY = 8) then iTempPosY := 0 else
                 if (iTempPosX = 4) and (iTempPosY = 5) then iTempPosY := 8 else
                 if (iTempPosX = 4) and (iTempPosY = 8) then iTempPosY := 7 else
                   Inc(iTempPosY);
            end; // case..of

            // checks for collisions with wolves, or eating of worms, or laying eggs
            for i := 1 to iMaxWolf do
              if (iTempPosX = rWolf[i].iPosX) and (iTempPosY = rWolf[i].iPosY) and (rWolf[i].iStatus = 10) then
                bMistake := true;

            if bMistake then
            // mistake has occured
              begin
                Gam_Stop_All_Timers;

                if not ((iTempPosX = rChicky.iPosX + 1) and (iTempPosX > 1)) then // updates x position of chicky if coming from left (except when x = 1)
                  rChicky.iPosX := iTempPosX;

                rChicky.iPosY := iTempPosY;                                       // updates y position of chicky, regardless of where he comes from

                iLoopGeneric := 1;
                Timer_Miss.Interval := 100;
                Timer_Miss.Enabled := true;
              end
            else
              // no mistake has occured
              begin
                i := 1;
                iTempScore := 0;

                while (i <= iMaxWolf) do
                  begin
                    // there's one or more escaping wolves in that place!
                    if (iTempPosX = rWolf[i].iPosX) and (iTempPosY = rWolf[i].iPosY) and (rWolf[i].iStatus = 11) then                                         // escaping wolf: wolf has just been eaten
                      begin
                        Inc(iTempScore, iScoreWolf);                                      // increases "temp" score by 200/400 points
                        iScoreWolf := 400;                                                // next wolf is worth 400 points
                        aIm_wolf[rWolf[i].iPosX, rWolf[i].iPosY].Visible := false;        // hides wolf
                        rWolf[i].iStatus := 0;                                            // wolf goes back into house

                        // power pill ends if there's no more wolves in iStatus = 11
                        bPowerPillcontinues := false;

                        for j := 1 to iMaxWolf do
                          if rWolf[j].iStatus = 11 then
                            bPowerPillcontinues := true;

                        if not bPowerPillcontinues then
                          begin
                            iLoopPowerPill := 0;
                            Timer_Power_Pill.Enabled := false;
                          end;
                    end;

                    // update position of chicky
                    rChicky.iPosX := iTempPosX;
                    rChicky.iPosY := iTempPosY;

                    Inc(i);
                  end;

                // lay eggs
                if (rChicky.iDirection <> 0) and (aLabyrinth[rChicky.iPosX, rChicky.iPosY].iContent = 1) then
                  begin
                    aLabyrinth[rChicky.iPosX, rChicky.iPosY].iContent := 0;
                    aIm_egg[rChicky.iPosX, rChicky.iPosY].Visible := true;
                    Inc(iTempScore, 10);
                  end;

                // eat worms
                if (rChicky.iDirection <> 0) and (aLabyrinth[rChicky.iPosX, rChicky.iPosY].iContent = 2) then
                  begin
                    aLabyrinth[rChicky.iPosX, rChicky.iPosY].iContent := 0;
                    aIm_egg[rChicky.iPosX, rChicky.iPosY].Visible := false;
                    Inc(iTempScore, 50);
                    iScoreWolf := 200;                                          // first wolf is worth 200 points

                    // puts attacking wolves outside house, into escaping mode
                    for i := 1 to iMaxWolf do
                      if rWolf[i].iStatus = 10 then
                        begin
                          rWolf[i].iStatus := 11;
                          iLoopPowerPill := 1;
                        end;

                    // if, in a result, there's any wolf that escapes, "power pill" timer starts ticking
                    if iLoopPowerPill = 1 then
                      begin
                        Timer_Power_Pill.Interval := 1;
                        Timer_Power_Pill.Enabled := true;
                      end;
                  end;

                // increases score with eggs, worms, eaten wolves, etc.
                if iTempScore > 0 then
                  begin
                    Inc(iScore, iTempScore);
                    Vid_Show_Panel_Score(iScore, true);
                  end;

                // play which sound sample? (part 1)
                pfSound := nil;

                if ((iTempScore = 10) or (iTempScore = 50)) and (iLoopPowerPill = 0) then // an egg was laid or worm was eaten, and it's not "power pill" time
                  pfSound := pfs_move
                else
                  if iTempScore > 50 then                                       // en escaping wolf has just been eaten
                    pfSound := pfs_bonus;

                // increases lives left if score > bonus score
                if iScore >= iScoreBonus then
                  begin
                    Inc(iLivesLeft, 1);
                    Vid_Show_Panel_Lives(iLivesLeft);
                    iScoreBonus := 999999;                                      // prevents from adding new bonus life
                    pfSound := pfs_bonus;
                  end;

                // play which sound sample? (part 2)
                if pfSound <> nil then
                  Snd_Play(pfSound);

                // count cells to clear
                iCount := 0;

                for i := 1 to 5 do
                  for j := 1 to 7 do
                    if aLabyrinth[i, j].iContent > 0
                      then Inc(iCount);

                if iCount = 0 then
                  begin                                                         // level complete!
                    Gam_Stop_All_Timers;

                    iLoopGeneric := 1;
                    Timer_Stage_Complete.Interval := 100;
                    Timer_Stage_Complete.Enabled := true;
                  end;

              end; // not bMistake

            // rendering chicky (regardless of whether a mistake has just occured)
            Vid_Show_All_Chicky(false);
            Vid_Show_Chicky(true);
          end
        else
          rChicky.iDirection := 0;                                              // chicky stops walking if cell is at a road end
      end // Odd(iLoop)
  else
{ wolves turn                                                                  }
    begin
      // checks for collision -> mistake
      for i := 1 to iMaxWolf do
        begin
          // checks whether chicky is beside wolf
          if (rWolf[i].iStatus = 10) then
            if ((aLabyrinth[rWolf[i].iPosX, rWolf[i].iPosY].Wolf.iAllowedDirections and 1) = 1) // up to wolf
            and (rWolf[i].iPosX = rChicky.iPosX) and (rWolf[i].iPosY = rChicky.iPosY + 1) then
              begin
                bMistake := true;
                Dec(rWolf[i].iPosY);
              end
            else
              if ((aLabyrinth[rWolf[i].iPosX, rWolf[i].iPosY].Wolf.iAllowedDirections and 2) = 2) // checks left to wolf
              and (rWolf[i].iPosX = rChicky.iPosX + 1) and (rWolf[i].iPosY = rChicky.iPosY) then
                bMistake := true
              else
                if ((aLabyrinth[rWolf[i].iPosX, rWolf[i].iPosY].Wolf.iAllowedDirections and 4) = 4) // checks right to wolf
                and (rWolf[i].iPosX = rChicky.iPosX - 1) and (rWolf[i].iPosY = rChicky.iPosY) then
                  begin
                    bMistake := true;
                    Inc(rWolf[i].iPosX);
                  end
                else
                  if ((aLabyrinth[rWolf[i].iPosX, rWolf[i].iPosY].Wolf.iAllowedDirections and 8) = 8) // checks down to wolf
                  and (rWolf[i].iPosX = rChicky.iPosX) and (rWolf[i].iPosY = rChicky.iPosY - 1) then
                    begin
                      bMistake := true;
                      Inc(rWolf[i].iPosY);
                    end;

          // checks whether wolf is about to go out of house, and chicky is in front of house
          if (rWolf[i].iStatus = 9) and (rChicky.iPosX = 3) and (rChicky.iPosY = 3) then
            begin
              bMistake := true;
              rWolf[i].iPosX := 3;
              rWolf[i].iPosY := 3;
              rWolf[i].iStatus := 10;
              im_wolf_3_4.Visible := false;
            end;
        end;

      if not bMistake then                                                      // no mistake has occured
        begin

          // movement
          for i := 1 to iMaxWolf do
            if (rWolf[i].iStatus = 10)                                          // wolf is attacking and iLoop = 2 or 4
            or ( (rWolf[i].iStatus = 11) and (iLoop = 4) ) then                 // wolf is escaping and iLoop = 4 only
              begin
                if (aLabyrinth[rWolf[i].iPosX, rWolf[i].iPosY].Wolf.iAllowedDirections and rWolf[i].iDirection) <> rWolf[i].iDirection then // dead end
                  rWolf[i].iDirection := 0;

                bWolfDirDone := false;

                repeat
                  iTempDir := trunc(Power(2, random(4)));                       // random 1/2/4/8

                  if (aLabyrinth[rWolf[i].iPosX, rWolf[i].iPosY].Wolf.iAllowedDirections and iTempDir) = iTempDir then
                    bWolfDirDone := true;

                  if rWolf[i].iDirection * iTempDir = 8 then                    // equals to rWolf[i].idirection opposite to iTempDir (1 vs 8, 2 vs 4, etc.)
                    bWolfDirDone := false;
                until
                  bWolfDirDone;

                rWolf[i].iDirection := iTempDir;

                case rWolf[i].iDirection of
                  1: Dec(rWolf[i].iPosY);
                  2: Dec(rWolf[i].iPosX);
                  4: Inc(rWolf[i].iPosX);
                  8: Inc(rWolf[i].iPosY);
                end; // case..of
              end;

          // wolf generate from house (iStatus = 0-9)
          iTempWolf := 0;                                                       // there's currently no wolves in house
          iTempWolfStatus := -1;                                                // set below 0, for use in next "if"

          for i := 1 to iMaxWolf do                                             // checks which wolf is in house (it there's one!), and evaluates his iStatus
            if (rWolf[i].iStatus > iTempWolfStatus) and (rWolf[i].iStatus < 10) then
              begin
                iTempWolf := i;
                iTempWolfStatus := rWolf[i].iStatus;
              end;

          if iTempWolf > 0 then                                                 // wolf moves in house
            begin
              Inc(rWolf[iTempWolf].iStatus);

              case rWolf[iTempWolf].iStatus of
                2: begin im_wolf_3_4.Visible := true;  im_wolf_3_4b.Visible := false; end;
                3: begin im_wolf_3_4.Visible := false; im_wolf_3_4b.Visible := true; end;
                4: begin im_wolf_3_4.Visible := true;  im_wolf_3_4b.Visible := false; end;
                5: begin im_wolf_3_4.Visible := false; im_wolf_3_4b.Visible := true; end;
                6: begin im_wolf_3_4.Visible := true;  im_wolf_3_4b.Visible := false; end;
                7: begin im_wolf_3_4.Visible := false; im_wolf_3_4b.Visible := true; end;
                8: begin im_wolf_3_4.Visible := true;  im_wolf_3_4b.Visible := false; end;
                9: begin im_wolf_3_4.Visible := true;  im_wolf_3_4b.Visible := false; end;
              end; // case..of

              if rWolf[iTempWolf].iStatus = 10 then                             // wolf exits from house and attacks
                begin
                  im_wolf_3_4.Visible := false;
                  rWolf[iTempWolf].iPosX := 3;
                  rWolf[iTempWolf].iPosY := 3;
                  rWolf[iTempWolf].iDirection := 2* trunc(Power(2, random(2))); // random 2 or 4
                end;
            end;
        end; // not bMistake

      // hide & render wolves
      for i := 1 to 5 do
        for j := 1 to 7 do
          if not ( (i = 3) and (j = 4) ) then
            aIm_wolf[i, j].Visible := false;

      for i := 1 to iMaxWolf do
        if (rWolf[i].iStatus = 10)                                              // wolf attacks (every tick)
          or ( (rWolf[i].iStatus = 11) and (iLoop = 4) ) then                   // wolf escapes (1 tick out of 2)
          aIm_wolf[rWolf[i].iPosX, rWolf[i].iPosY].Visible := true;

      // has escaping wolf gone over chicky?
      if not bMistake then
        for i := 1 to iMaxWolf do
          if (rWolf[i].iStatus = 11) and (iLoop = 4) then                       // wolf escapes (1 tick out of 2)
            if (rWolf[i].iPosX = rChicky.iPosX) and (rWolf[i].iPosY = rChicky.iPosY) then
              begin
                // wolf was eaten and disappears
                Inc(iScore, iScoreWolf);                                        // increases score by 200/400 points
                iScoreWolf := 400;                                              // next wolf is worth 400 points
                Vid_Show_Panel_Score(iScore, true);                             // displays updated score
                aIm_wolf[rWolf[i].iPosX, rWolf[i].iPosY].Visible := false;      // hides wolf
                rWolf[i].iStatus := 0;                                          // wolf goes back into house

                // increases lives left if score > bonus score
                if iScore >= iScoreBonus then
                  begin
                    Inc(iLivesLeft, 1);
                    Vid_Show_Panel_Lives(iLivesLeft);
                    iScoreBonus := 999999;                                      // prevents from adding new bonus life
                  end;

                // power pill ends if there's no more wolves in iStatus = 11
                bPowerPillcontinues := false;

                for j := 1 to iMaxWolf do
                  if rWolf[j].iStatus = 11 then
                    bPowerPillcontinues := true;

                if not bPowerPillcontinues then
                  begin
                    iLoopPowerPill := 0;
                    Timer_Power_Pill.Enabled := false;
                  end;

                Snd_Play(pfs_bonus);
              end;

      // mistake
      if bMistake then                                                          // mistake has occured
        begin
          Gam_Stop_All_Timers;

          iLoopGeneric := 1;
          Timer_Miss.Interval := 100;
          Timer_Miss.Enabled := true;
        end
    end; // Odd(iLoop) else

  // loops actions
  if not bMistake then                                                          // no mistake has occured
    begin
      // sets gates open/closed randomly
      if iLoop = 4 then
        begin
          iGate := Random(5) + 1;
          Gam_Set_Gate;
        end;

      // loops action
      Inc(iLoop);

      if iLoop = 5 then
        iLoop := 1;
    end;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ game is over: play "Game Over" tune, then freeze for 30 seconds, and finally }
{ change mode to "Time 1", if no button is pressed                             }
begin
  case iLoopGeneric of
    1: begin
         im_panel_game_over.Visible := true;
         Vid_Show_All_Wolf(false);

         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 3400;
       end;
    2: begin
         if (iMode = 6) and (iScore > iHiScoreA) then                           // records high score to system registry (game A)
           begin
             iHiScoreA := iScore;
             iReg.WriteInteger('', 'HiScore1', iHiScoreA);
           end;

         if (iMode = 7) and (iScore > iHiScoreB) then                           // records high score to system registry (game B)
           begin
             iHiScoreB := iScore;
             iReg.WriteInteger('', 'HiScore2', iHiScoreB);
           end;

         Dec(iMode, 4);                                                         // goes back into "select" mode (game A or game B)
         Gam_Set_Mode;

         Timer_Game_Over.Interval := 30000;
       end;
    3: begin                                                                    // back to "demo" mode
         iMode := 1;
         Gam_Set_Mode;

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGeneric);
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ sets Game A or Game B modes after Game A or Game B buttons are released      }
begin
  if (iMode = 4) or (iMode = 5) then                                            // HiScore A or HiScore B ->  Game A or Game B
    begin
      Inc(iMode, 2);
      Gam_Set_Mode;
    end;

  Timer_Game_Start.Enabled := false;
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occurred. plays sound sample and then either returns to game or   }
{ game is over.                                                                }
begin
  case iLoopGeneric of
    1: Timer_Miss.Interval := 100;
    2: begin
         Snd_Play(pfs_miss);
         Vid_Show_Chicky(false);

         Timer_Miss.Interval := 240;
       end;
    3: Vid_Show_Chicky(true);
    4: Vid_Show_Chicky(false);
    5: begin
         Vid_Show_Chicky(true);

         Timer_Miss.Interval := 750;
       end;
    6: begin
         Vid_Show_Chicky(false);

         Timer_Miss.Interval := 1000;
       end;
    7: begin
         Dec(iLivesLeft);

         if iLivesLeft < 0 then
           begin                                                                // game over
             iLoopGeneric := 0; {turns to 1 immediately}
             Timer_Game_Over.Interval := 1;
             Timer_Game_Over.Enabled := true;
           end
         else                                                                   // game reprise
           begin
             iLoopGeneric := 0; {turns to 1 immediately}
             Timer_Stage_Start.Interval := 1;
             Timer_Stage_Start.Enabled := true;
           end;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGeneric);
end;





procedure TForm1.Timer_Power_PillTimer(Sender: TObject);
{ power pill routine                                                           }
  var
    i: integer;
begin
  Timer_Power_Pill.Interval := 250;

  if Odd(iLoopPowerPill) then
    Snd_Play(pfs_power1)
  else
    Snd_Play(pfs_power2);

  Inc(iLoopPowerPill);

  // power pill ends its effectiveness
  if iLoopPowerPill > iMaxPowerPill then
    begin
      // escaping wolves turn back to attack
      for i := 1 to iMaxWolf do
        if rWolf[i].iStatus = 11 then
          rWolf[i].iStatus := 10;

      iLoopPowerPill := 0;
      Timer_Power_Pill.Enabled := false;
    end;
end;





procedure TForm1.Timer_Stage_CompleteTimer(Sender: TObject);
{ stage is complete. plays fanfare and then returns to game                    }
  var
    i, j: integer;
begin
  case iLoopGeneric of
    1: begin
         Vid_Show_All_Wolf(false);
         Snd_Play(pfs_complete);
         Inc(iLoopGeneric);
         Timer_Stage_Complete.Interval := 2450;
       end;
    2: begin
         for i := 0 to 6 do
           for j := 0 to 8 do
             aLabyrinth[i, j].iContent := 0;

         for i := 1 to 5 do
           for j := 1 to 7 do
             aLabyrinth[i, j].iContent := 1;

         aLabyrinth[2, 6].iContent := 2;
         aLabyrinth[4, 2].iContent := 2;
         aLabyrinth[3, 4].iContent := 0;

         Vid_Show_All_Chicky(false);                                            // hides all items, chars, etc.
         Vid_Show_All_Egg(false);
         aIm_egg[2, 6].Visible := true; aIm_egg[4, 2].Visible := true;          // displays worms
         Inc(iStage); Gam_Set_Difficulty;

         iLoopGeneric := 1;                                                     // initializes "Stage Start" timer
         Timer_Stage_Start.Interval := 1;
         Timer_Stage_Start.Enabled := true;                                     // starts "Game Start" timer

         Timer_Stage_Complete.Enabled := false;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Stage_StartTimer(Sender: TObject);
{ plays "game start" tune, initialises new maze and game starts                }
  var
    i: integer;
begin
  case iLoopGeneric of
    1: begin
         Snd_Play(pfs_stgstart);
         Vid_Show_Panel_Lives(iLivesLeft);
         Vid_Show_All_Wolf(false);
         Inc(iLoopGeneric);

         Timer_Stage_Start.Interval := 700;
       end;
    2: begin
         for i := 1 to iMaxWolf do
           rWolf[i].iStatus := 0;

         rChicky.iPosX := 3;
         rChicky.iPosY := 5;
         rChicky.iDirection := 0;
         Vid_Show_Chicky(true);
         iGate := 1; Gam_Set_Gate;                                              // all gates open at level start

         iLoop := 1;                                                            // initializes game loop
         iLoopPowerPill := 0;                                                   // disables "power pill" mode
         Timer_Game.Enabled := true;                                            // game begins
         Timer_Stage_Start.Enabled := false;
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
